/*
 * Copyright (c) 1990 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

#include <stdio.h>
#include "dixie.h"

#include <quipu/attrvalue.h>
#include <quipu/ds_error.h>
#include <quipu/commonarg.h>
#include <quipu/remove.h>
#include <quipu/dua.h>

#define NULL	0

#define REMOVE_END(x)	{ dn_free(thedn); \
			return(x); }

extern int	debug;
extern int	bound;
extern int	stream_connected;
extern short	timelimit, sizelimit;

/* args for the remove call */
struct ds_removeentry_arg	remove_arg = { default_common_args };
struct DSError			remove_error;

/* ARGSUSED */
removeop( bp, rp, rpsize, dontderef )
char	*bp, *rp;
int	rpsize;
char	dontderef;
{
	DN		thedn = NULLDN;
	int		i;

	remove_arg.rma_common.ca_servicecontrol.svc_timelimit = timelimit;
	remove_arg.rma_common.ca_servicecontrol.svc_sizelimit = sizelimit;
	if ( dontderef )
		remove_arg.rma_common.ca_servicecontrol.svc_options |=
		    SVC_OPT_DONTDEREFERENCEALIAS;
	else
		remove_arg.rma_common.ca_servicecontrol.svc_options &=
		    ~SVC_OPT_DONTDEREFERENCEALIAS;
	*rp = '\0';
	if ( debug & 2 ) printf("Remove (%s)\n", bp);

	/* 
	 * First convert the dn given into a DN structure
	 */

	if ( *bp == '@' ) bp++;
	if ( (thedn = str2dn( bp )) == NULLDN ) {
		if ( debug > 1 ) printf("removeop: str2dn (%s) failed\n",bp);
		return( DXI_BADDN );
	}
	remove_arg.rma_object = thedn;

	/* do the actual remove */
	for ( i = 0; i < 2; i++ ) {
		if ( ds_removeentry( &remove_arg, &remove_error ) == DS_OK )
			break;
		if ( debug & 2 ) printf("ds_remove failed\n");

		if ( remove_error.dse_type == DSE_REFERRAL )
			if ( chase_referral( &remove_error ) != OK )
				break;
			else
				continue;

		if ( remove_error.dse_type != DSE_REMOTEERROR
		    || stream_connected )
			break;

		(void) nullbind();

		if ( bound == 0 )
			break;
	}

	if ( debug & 2 )
		printf("remove successful - checking for errors\n");

	switch ( remove_error.dse_type ) {
	case DSE_REMOTEERROR:
		if ( debug & 2 ) {
			printf("\tservice error\n");
			error( &remove_error );
		}
		REMOVE_END( DXI_DSA_DOWN );
	case DSE_REFERRAL:
		if ( debug & 2 ) {
			printf("\treferral error\n");
			error( &remove_error );
		}
		REMOVE_END( DXI_REFER_ERROR );
	case DSE_NAMEERROR:
		if ( debug & 2 ) {
			printf("\tname error\n");
			error( &remove_error );
		}
		REMOVE_END( DXI_NAME_ERROR );
	case DSE_SERVICEERROR:
		if ( debug & 2 ) {
			printf("\tservice error\n");
			error( &remove_error );
		}
		if (remove_error.dse_un.dse_un_service.DSE_sv_problem
		    != DSE_SV_TIMELIMITEXCEEDED &&
		    remove_error.dse_un.dse_un_service.DSE_sv_problem
		    != DSE_SV_ADMINLIMITEXCEEDED)
			REMOVE_END( DXI_ERROR )
		break;
	case DSE_NOERROR:
		if ( debug & 2 ) printf("\tno error\n");
		break;
	default:
		if ( debug & 2 ) {
			printf("\tsome unknown error (%d)\n");
			error( &remove_error );
		}
		REMOVE_END( DXI_ERROR );
	}

	REMOVE_END( DXI_SUCCESS );
}
