/*
 * Copyright (c) 1991, 1992 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <dixie.h>
#include "ud.h"

char me[BUFSIZ];		/* the user's set name */
char password[512];		/* password entered my user */
char bound_as[BUFSIZ];		/* the user's bound name */
int verbose;			/* verbosity indicator */


bind_info()
{
	printf("  Bound as \"%s\"\n", bound_as);
}

auth(who, best_guess)
char *who, *best_guess;
{
	int rc;			/* return code from dxi_bind() */
	char *passwd;		/* returned by mygetpass() */
	char *dn;		/* distinghished name from dxi_getdn() */
	char *cp;		/* miscellaenous pointer */
	static char prompt[BUFSIZ];	/* place for us to sprintf the prompt */
	static char name[BUFSIZ];	/* place to store the user's name */
	extern struct entry Person;	/* look here for a name if needed */
	extern char *mygetpass();	/* getpass() passwds are too short */
	extern char *find();		/* for looking up 'name' */

	if (who == NULL) {
		/*
		 *  The user needs to bind.  If <who> is not specified, we
		 *  can make some guesses.  If the user has used the 'iam'
		 *  command, then try the name s/he set.  If not, try the
		 *  name retrieved with the last 'find' operation.  If no
		 *  such command has been issued, we don't have any guess,
		 *  and the user will have to supply a name.
		 */
		if (me[0] != '\0')
			who = me;
		else if (Person.name[0] != '\0')
			who = Person.name;
		else if (best_guess != NULL)
			who = best_guess;
		else
			who = "????";
		if (verbose) {
			printf("  You are about to be asked to authenticate to the X.500 server.\n");
			printf("  You need to authenticate to the server in order to be able to\n");
			printf("  make changes to entries stored in the server's database.\n");
			printf("  This program has tried to make a guess at who you might want to\n");
			printf("  authenticate as.  This guess appears in the square brackets.\n");
			printf("  If the guess is a good one, and the name that appears is the one\n");
			printf("  you wish to use, simply hit the RETURN key.  If the guess is a poor\n");
			printf("  one, type in the name you wish to use.\n\n");
		}
		for (;;) {
			printf("  Authenticate as [%s]? ", who);
			fflush(stdout);
			fetch_buffer(name, sizeof(name), stdin);
			if (name[0] != '\0')
				who = name;
			if (strcmp(who, "????"))
				break;
		}
	}

	/*
	 *  Bail out if the name is bogus.  If not, strip off the junk
	 *  at the start of the DN, build a prompt, and get a password 
	 *  from the user.  Then perform the dxi_bind().
	 */
	if ((cp = find(who)) == NULL)
		return(0);
	dn = dxi_getdn(dxi_firstentry(cp));
	cp = strrchr(dn, '=');
	cp++;
	sprintf(prompt, "  Enter password for \"%s\": ", cp);
	do {
		passwd = mygetpass(prompt);
	} while (*passwd == '\0');
	rc = dxi_bind(dn, passwd);
	if (rc == -1) {
		if (dxi_errno == DXI_NEX_ATTR)
			fprintf(stderr, "  Entry has no password\n");
		else if (dxi_errno == DXI_REFUSED)
			fprintf(stderr, "  Password does not match\n");
		else
			dxi_perror( "dxi_bind" );
		return(-1);
	}
	else if (verbose)
		printf("  Authentication successful.\n\n");
	else
		printf("\n");
	(void) strcpy(bound_as, cp);
	(void) strcpy(password, passwd);
	return(0);
}

iam(who)
char *who;
{
	if (!strcmp(who, "me"))
		printf("  Of course you are.\n");
	else {
		(void) strcpy(me, who);
		print_my_name();
	}
}

print_my_name()
{
	if (me[0] == '\0')
		printf("  Beats me\n");
	else
		printf("  You are \"%s\"\n", me);
}
